package com.cultural.heritage.controller.book;


import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.bookingDate.BookingDateAddRequest;
import com.cultural.heritage.model.dto.bookingTime.BookingTimeAddRequest;
import com.cultural.heritage.model.dto.photoProducts.PhotoProductsAddRequest;
import com.cultural.heritage.model.dto.photoProducts.PhotoProductsQueryRequest;
import com.cultural.heritage.model.dto.photoProducts.PhotoProductsUpdateRequest;
import com.cultural.heritage.model.entity.AdvanceOrder;
import com.cultural.heritage.model.entity.BookingDate;
import com.cultural.heritage.model.entity.BookingTime;
import com.cultural.heritage.model.entity.PhotoProducts;
import com.cultural.heritage.model.vo.bookingDate.BookingDateVO;
import com.cultural.heritage.model.vo.bookingTime.BookingTimeVO;
import com.cultural.heritage.model.vo.photoProducts.PhotoProductsMainInfoVO;
import com.cultural.heritage.model.vo.photoProducts.PhotoProductsVO;
import com.cultural.heritage.service.book.AdvanceOrderService;
import com.cultural.heritage.service.book.BookingDateService;
import com.cultural.heritage.service.book.BookingTimeService;
import com.cultural.heritage.service.book.PhotoProductsService;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.utils.DecoderUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/photoProducts")
@Slf4j
@Tag(name = "写真产品管理模块")
public class PhotoProductsController {
    

    @Resource
    private PhotoProductsService photoProductsService;



    @Resource
    private BookingDateService bookingDateService;



    @Resource
    private BookingTimeService bookingTimeService;



    @Resource
    private CommonService commonService;



    @Resource
    private AdvanceOrderService advanceOrderService;



    @Resource
    private RedisTemplate redisTemplate;


    /**
     * Web端管理员添加写真产品
     * @param photoProductsAddRequest 写真产品添加请求体
     * @return  写真产品id
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员添加写真产品", description = "参数：写真产品添加请求体，权限：管理员(admin, boss)，方法名：addPhotoProducts")
    public BaseResponse<Long> addPhotoProducts(@RequestBody PhotoProductsAddRequest photoProductsAddRequest) {
        if (photoProductsAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        PhotoProducts photoProducts = new PhotoProducts();
        BeanUtils.copyProperties(photoProductsAddRequest, photoProducts);
        // 校验
        photoProductsService.validPhotoProducts(photoProducts, false);
        boolean result = photoProductsService.save(photoProducts);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真产品添加失败");

        // 添加预约日期
        List<BookingDateAddRequest> bookingDateAddRequestList = photoProductsAddRequest.getBookingDateAddRequestList();
        List<BookingDate> bookingDateList = bookingDateAddRequestList.stream().map(bookingDateAddRequest -> {
            BookingDate bookingDate = new BookingDate();
            BeanUtils.copyProperties(bookingDateAddRequest, bookingDate);
            bookingDate.setPhotoProductId(photoProducts.getId());
            // 校验
            bookingDateService.validBookingDate(bookingDate, false);
            return bookingDate;
        }).toList();

        boolean saveBatch = bookingDateService.saveBatch(bookingDateList);
        ThrowUtils.throwIf(!saveBatch, ErrorCode.OPERATION_ERROR, "写真产品预约日期添加失败");

        // 添加预约时间段
        List<BookingTime> bookingTimes = new ArrayList<>();
        for (int i = 0; i < bookingDateList.size(); i++) {
            BookingDate bookingDate = bookingDateList.get(i);
            Long bookingDateId = bookingDate.getId();
            BookingDateAddRequest bookingDateAddRequest = bookingDateAddRequestList.get(i);
            List<BookingTimeAddRequest> bookingTimeAddRequestList = bookingDateAddRequest.getBookingTimeAddRequestList();
            List<BookingTime> bookingTimeList = bookingTimeAddRequestList.stream().map(bookingTimeAddRequest -> {
                BookingTime bookingTime = new BookingTime();
                BeanUtils.copyProperties(bookingTimeAddRequest, bookingTime);
                bookingTime.setBookingDateId(bookingDateId);
                return bookingTime;
            }).toList();
            bookingTimes.addAll(bookingTimeList);
        }
        boolean save = bookingTimeService.saveBatch(bookingTimes);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "写真产品预约时间添加失败");

        return ResultUtils.success(photoProducts.getId());
    }




    /**
     * Web管理员更新写真产品
     * @param photoProductsUpdateRequest  写真产品更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员更新写真产品", description = "参数：服装更新请求体，权限：管理员(admin, boss)，方法名：updatePhotoProducts")
    public BaseResponse<Boolean> updatePhotoProducts(@RequestBody PhotoProductsUpdateRequest photoProductsUpdateRequest) {
        if (photoProductsUpdateRequest == null || photoProductsUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        PhotoProducts photoProducts = new PhotoProducts();
        BeanUtils.copyProperties(photoProductsUpdateRequest, photoProducts);
        // 校验
        photoProductsService.validPhotoProducts(photoProducts, true);
        boolean result = photoProductsService.updateById(photoProducts);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真产品更新失败");
        return ResultUtils.success(true);
    }




    /**
     * Web管理员删除写真产品
     * @param commonRequest  写真产品id
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员删除写真产品", description = "参数：写真产品id，权限：管理员(admin, boss)，方法名：deletePhotoProducts")
    public BaseResponse<Boolean> deletePhotoProducts(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        // 获取预约日期列表
        List<BookingDate> bookingDateList = commonService.findByFieldEqTargetField("photoProductId", id, bookingDateService);
        // 获取预约时间列表
        List<BookingTime> bookingTimeList = commonService.findByFieldInTargetField(bookingDateList, bookingTimeService, BookingDate::getId, "bookingDateId");
        // 删除写真产品下的预约时间
        bookingTimeService.removeBatchByIds(bookingTimeList);
//        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "预约时间批量删除失败");
        // 删除写真产品下的预约日期
        bookingDateService.removeBatchByIds(bookingDateList);
//        ThrowUtils.throwIf(!success, ErrorCode.OPERATION_ERROR, "预约日期批量删除失败");
        // 删除写真产品
        boolean result = photoProductsService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真产品删除失败");
        return ResultUtils.success(true);
    }




    /**
     * Web管理员批量删除写真产品
     * @param commonBatchRequest  写真产品id列表
     * @return  是否删除成功
     */
    @PostMapping("/delBatch")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员批量删除写真产品", description = "参数：写真产品id列表，权限：管理员(admin, boss)，方法名：deletePhotoProducts")
    public BaseResponse<Boolean> delBatchPhotoProducts(@RequestBody CommonBatchRequest commonBatchRequest) {
        if (commonBatchRequest == null || CollectionUtils.isEmpty(commonBatchRequest.getIdList())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        List<Long> idList = commonBatchRequest.getIdList();
        // 获取写真产品的预约日期
        List<BookingDate> bookingDateList = commonService.findByFieldInTargetField(idList, bookingDateService, id -> id, "photoProductId");
        // 获取写真产品的预约时间
        List<BookingTime> bookingTimeList = commonService.findByFieldInTargetField(bookingDateList, bookingTimeService, BookingDate::getId, "bookingDateId");
        // 删除写真产品下的预约时间
        bookingTimeService.removeBatchByIds(bookingTimeList);
//        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "预约时间批量删除失败");
        // 删除写真产品下的预约日期
        bookingDateService.removeBatchByIds(bookingDateList);
//        ThrowUtils.throwIf(!success, ErrorCode.OPERATION_ERROR, "预约日期批量删除失败");
        // 批量删除写真产品
        boolean result = photoProductsService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真产品批量删除失败");
        return ResultUtils.success(true);
    }




    /**
     * Web管理员分页查询写真产品
     * @param photoProductsQueryRequest   写真产品查询请求体
     * @return  写真产品列表
     */
    @PostMapping("/list/page")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员分页查询写真产品", description = "参数：写真产品查询请求体，权限：管理员(admin, boss)，方法名：listPhotoProductsByPage")
    public BaseResponse<Page<PhotoProductsVO>> listPhotoProductsByPage(@RequestBody PhotoProductsQueryRequest photoProductsQueryRequest) {
        if (photoProductsQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = photoProductsQueryRequest.getCurrent();
        long pageSize = photoProductsQueryRequest.getPageSize();
        QueryWrapper<PhotoProducts> queryWrapper = photoProductsService.getQueryWrapper(photoProductsQueryRequest);
        Page<PhotoProducts> page = photoProductsService.page(new Page<>(current, pageSize), queryWrapper);
        // 获取写真产品列表
        List<PhotoProducts> records = page.getRecords();
        // 获取预约日期表
        List<BookingDate> bookingDateList = commonService.findByFieldInTargetField(records, bookingDateService, PhotoProducts::getId, "photoProductId");
        // 获取预约时间表
        List<BookingTime> bookingTimeList = commonService.findByFieldInTargetField(bookingDateList, bookingTimeService, BookingDate::getId, "bookingDateId");
        // 封装成BookingTimeVO列表, 并将预约时间从小到大排序
        List<BookingTimeVO> bookingTimeVOList = commonService.convertList(bookingTimeList, BookingTimeVO.class);
        bookingTimeVOList.sort((b1, b2) -> {
            DateTime time1 = DateUtil.parse(b1.getTimePoint(), "HH:mm");
            DateTime time2 = DateUtil.parse(b2.getTimePoint(), "HH:mm");
            return time1.compareTo(time2);  // 按照时间升序排序
        });

        // 封装成BookingDateVO列表
        List<BookingDateVO> bookingDateVOS = new ArrayList<>();
        for (BookingDate bookingDate : bookingDateList) {
            Long bookingDateId = bookingDate.getId();
            List<BookingTimeVO> bookingTimeVOS = bookingTimeVOList.stream().filter(bookingTimeVO ->
                    bookingTimeVO.getBookingDateId().equals(bookingDateId)).toList();

            BookingDateVO bookingDateVO = new BookingDateVO();
            BeanUtils.copyProperties(bookingDate, bookingDateVO);
            bookingDateVO.setBookingTimeVOList(bookingTimeVOS);
            bookingDateVOS.add(bookingDateVO);
        }
        // 将预约日期从小到大排序
        bookingDateVOS.sort((app1, app2) -> {
            DateTime date1 = DateUtil.parse(app1.getSpecificDate(), "yyyy-MM-dd");
            DateTime date2 = DateUtil.parse(app2.getSpecificDate(), "yyyy-MM-dd");
            return date1.compareTo(date2);
        });

        // 封装成BookingPhotoProductsVO列表
        List<PhotoProductsVO> photoProductsVOList = new ArrayList<>();
        for (PhotoProducts photoProducts : records) {
            Long photoProductsId = photoProducts.getId();
            List<BookingDateVO> bookingDateVOList = bookingDateVOS.stream().filter(bookingDateVO ->
                    bookingDateVO.getPhotoProductId().equals(photoProductsId)).toList();

            PhotoProductsVO photoProductsVO = new PhotoProductsVO();
            BeanUtils.copyProperties(photoProducts, photoProductsVO);
            photoProductsVO.setBookingDateVOList(bookingDateVOList);
            photoProductsVOList.add(photoProductsVO);
        }

        Page<PhotoProductsVO> voPage = new Page<>();
        voPage.setRecords(photoProductsVOList);
        voPage.setTotal(page.getTotal());
        voPage.setCurrent(page.getCurrent());
        voPage.setSize(page.getSize());
        voPage.setPages(page.getPages());
        return ResultUtils.success(voPage);
    }



    /**
     * web端用户根据id查询写真产品主要信息
     * @param commonRequest 写真产品id
     * @return  写真产品
     */
    @PostMapping("/list/main/id")
    @Operation(summary = "web端用户根据id查询写真产品主要信息", description = "参数：写真产品id，权限：所有人，方法名：getPhotoProductsMainInfoById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<PhotoProductsMainInfoVO> getPhotoProductsMainInfoById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        PhotoProducts photoProducts = photoProductsService.getById(id);
        PhotoProductsMainInfoVO photoProductsMainInfoVO = new PhotoProductsMainInfoVO();
        BeanUtils.copyProperties(photoProducts, photoProductsMainInfoVO);
        photoProductsMainInfoVO.setRichText(DecoderUtils.decodeText(photoProductsMainInfoVO.getRichText()));
        return ResultUtils.success(photoProductsMainInfoVO);
    }






    /**
     * 小程序端用户根据id查询写真产品
     * @param commonRequest 写真产品id
     * @return  写真产品
     */
    @PostMapping("/list/detail")
    @Operation(summary = "小程序端用户根据id查询写真产品", description = "参数：写真产品id，权限：所有人，方法名：getPhotoProductsById")
    public BaseResponse<PhotoProductsVO> getPhotoProductsById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        // 获取所有写真预约类订单
        Map<String, Integer> isOccupiedMap = new HashMap<>();
        QueryWrapper<AdvanceOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderStatus", OrderStatusConstant.PENDING_PAYMENT).or().eq("orderStatus", OrderStatusConstant.PENDING_SHIPMENT);
        List<AdvanceOrder> advanceOrderList = advanceOrderService.list(queryWrapper);
        for (AdvanceOrder advanceOrder : advanceOrderList) {
            String specificDate = advanceOrder.getSpecificDate();
            String timePoint = advanceOrder.getTimePoint();
            isOccupiedMap.put(specificDate + "&" + timePoint, 1);
        }

        Long id = commonRequest.getId();
        PhotoProducts photoProducts = photoProductsService.getById(id);
        ThrowUtils.throwIf(photoProducts == null || photoProducts.getIsShelves() == 0, ErrorCode.OPERATION_ERROR, "该商品已被删除或者已下架");
        // 获取预约日期表
        List<BookingDate> bookingDateList = commonService.findByFieldEqTargetField("photoProductId", id, bookingDateService);
        // 获取预约时间表
        List<BookingTime> bookingTimeList = commonService.findByFieldInTargetField(bookingDateList, bookingTimeService, BookingDate::getId, "bookingDateId");
        // 封装成BookingTimeVO列表, 并将预约时间从小到大排序
        List<BookingTimeVO> bookingTimeVOList = commonService.convertList(bookingTimeList, BookingTimeVO.class);
        bookingTimeVOList.sort((b1, b2) -> {
            DateTime time1 = DateUtil.parse(b1.getTimePoint(), "HH:mm");
            DateTime time2 = DateUtil.parse(b2.getTimePoint(), "HH:mm");
            return time1.compareTo(time2);  // 按照时间升序排序
        });

        // 封装成BookingDateVO列表
        List<BookingDateVO> bookingDateVOS = new ArrayList<>();
        for (BookingDate bookingDate : bookingDateList) {
            Long bookingDateId = bookingDate.getId();
            List<BookingTimeVO> bookingTimeVOS = bookingTimeVOList.stream().filter(bookingTimeVO ->
                    bookingTimeVO.getBookingDateId().equals(bookingDateId)).toList();

            BookingDateVO bookingDateVO = new BookingDateVO();
            BeanUtils.copyProperties(bookingDate, bookingDateVO);
            bookingDateVO.setBookingTimeVOList(bookingTimeVOS);
            bookingDateVOS.add(bookingDateVO);
        }

        // 将预约日期从小到大排序
        bookingDateVOS.sort((b1, b2) -> {
            DateTime date1 = DateUtil.parse(b1.getSpecificDate(), "yyyy-MM-dd");
            DateTime date2 = DateUtil.parse(b2.getSpecificDate(), "yyyy-MM-dd");
            return date1.compareTo(date2);
        });

        List<BookingDateVO> bookingDateVOList = new ArrayList<>();
        // 过滤掉不合理的预约时间
        for (BookingDateVO bookingDateVO : bookingDateVOS) {
            // 过滤掉没有空闲的预约日期
            if (bookingDateVO.getIsAvailable() == 0) continue;
            // 过滤掉已过期的预约日期
            String specificDate = bookingDateVO.getSpecificDate();
            String today = DateUtil.today();
            int result = DateUtil.compare(DateUtil.parse(specificDate), DateUtil.parse(today));
            if (result < 0) continue;
            // 如果预约日期相同，过滤掉预约时间以及被占用的时间
            List<BookingTimeVO> bookingTimeVOS = bookingDateVO.getBookingTimeVOList();
            if (result == 0) {
                bookingTimeVOS = bookingTimeVOS.stream().filter(bookingTimeVO -> {
                    String point = bookingTimeVO.getTimePoint();
                    String timePoint = point + ":00";
                    DateTime targetTime = DateUtil.parse(timePoint, "HH:mm:ss");
                    DateTime now = DateUtil.date();
                    DateTime currentTime = DateUtil.parse(now.toString("HH:mm:ss"), "HH:mm:ss");
                    return currentTime.compareTo(targetTime) < 0 && isOccupiedMap.get(specificDate + "&" + point) == null;
                }).toList();
            } else {
                bookingTimeVOS = bookingTimeVOS.stream().filter(bookingTimeVO -> {
                    String timePoint = bookingTimeVO.getTimePoint();
                    return isOccupiedMap.get(specificDate + "&" + timePoint) == null;
                }).toList();
            }
            if (bookingTimeVOS.isEmpty()) continue;
            bookingDateVO.setBookingTimeVOList(bookingTimeVOS);
            bookingDateVOList.add(bookingDateVO);
        }

        // 封装成BookingPhotoProductsVO列表
        PhotoProductsVO photoProductsVO = new PhotoProductsVO();
        BeanUtils.copyProperties(photoProducts, photoProductsVO);
        photoProductsVO.setRichText(DecoderUtils.decodeText(photoProductsVO.getRichText()));
        photoProductsVO.setBookingDateVOList(bookingDateVOList);

        return ResultUtils.success(photoProductsVO);
    }





    /**
     * Web端管理员上(下)架写真预约产品
     * @param commonRequest 写真产品id
     * @return 是否更新成功
     */
    @PostMapping("/shelves")
    @Operation(summary = "Web端管理员上(下)架写真预约产品", description = "参数：写真产品id，权限：管理员(admin, boss)，方法名：updatePhotoProductsShelvesStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updatePhotoProductsShelvesStatus(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 获取当前服务类商品的上(下)架状态
        Long id = commonRequest.getId();
        PhotoProducts photoProducts = photoProductsService.getById(id);
        ThrowUtils.throwIf(photoProducts == null, ErrorCode.OPERATION_ERROR, "写真产品不存在");
        Integer status = photoProducts.getIsShelves() == 0 ? 1 : 0;
        UpdateWrapper<PhotoProducts> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isShelves", status);
        boolean update = photoProductsService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "上架状态更新失败");
        return ResultUtils.success(true);
    }




    /**
     * Web端管理员获取Map集合(键：编号（从1开始）， 值：写真产品id）
     * @return map集合
     */
    @PostMapping("/list/map")
    @Operation(summary = "Web端管理员获取Map集合(键：编号（从1开始）， 值：写真产品id）", description = "参数：无，权限：管理员(admin, boss)，方法名：queryPhotoProductsMap")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Map<Integer, Long>> queryPhotoProductsMap() {
        List<PhotoProducts> photoProductsList = photoProductsService.list();
        Collections.reverse(photoProductsList);
        Map<Integer, Long> map = new HashMap<>();
        int cnt = 1;
        for (PhotoProducts photoProducts : photoProductsList) {
            Long id = photoProducts.getId();
            Integer isShelves = photoProducts.getIsShelves();
            if (isShelves == 1) {
                map.put(cnt, id);
            }
            cnt ++;
        }
        return ResultUtils.success(map);
    }




    /**
     * Web端管理员保存特殊写真产品指向的id
     * @return 是否保存成功
     */
    @PostMapping("/special/save")
    @Operation(summary = "Web端管理员保存特殊写真产品指向的id", description = "参数：无，权限：管理员(admin, boss)，方法名：saveSpecialPhotoProductById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> saveSpecialPhotoProductById(@RequestBody CommonBatchRequest commonBatchRequest) {
        if (commonBatchRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        List<Long> idList = commonBatchRequest.getIdList();
        redisTemplate.opsForValue().set("photoProductIds", idList);
        return ResultUtils.success(true);
    }



    /**
     * Web端（小程序端）管理员查询特殊写真产品指向的id
     * @return 写真产品id列表
     */
    @PostMapping("/special/get")
    @Operation(summary = "Web端管理员（小程序端）查询特殊写真产品指向的id", description = "参数：写真产品id列表，权限：管理员(admin, boss)，方法名：getSpecialPhotoProductIds")
    public BaseResponse<List<Long>> getSpecialPhotoProductIds () {

        Object valueObj = redisTemplate.opsForValue().get("photoProductIds");
        List<Long> newIds = new ArrayList<>();
        if (valueObj != null) {
            List<Long> ids = ((List<?>) valueObj).stream()
                    .map(obj -> ((Number) obj).longValue())
                    .collect(Collectors.toList());
            for (Long id: ids) {
                PhotoProducts photoProducts = photoProductsService.getById(id);
                if (photoProducts == null || photoProducts.getIsShelves() == 0) {
                    newIds.add(0L);
                } else {
                    newIds.add(id);
                }
            }
        } else {
            newIds = Arrays.asList(0L, 0L);
        }
        redisTemplate.opsForValue().set("photoProductIds", newIds);
        return ResultUtils.success(newIds);
    }



}
