package com.cultural.heritage.controller.good;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.CommonStringRequest;
import com.cultural.heritage.model.dto.coupon.*;
import com.cultural.heritage.model.entity.Coupon;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.entity.UserCoupon;
import com.cultural.heritage.model.vo.coupon.CouponVO;
import com.cultural.heritage.model.vo.coupon.UserCouponVO;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.good.CouponService;
import com.cultural.heritage.service.good.UserCouponService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("/coupon")
@Slf4j
@Tag(name = "优惠券管理模块")
public class CouponController {

    @Resource
    private CouponService couponService;


    @Resource
    private UserService userService;


    @Resource
    private UserCouponService userCouponService;


    @Resource
    private CommonService commonService;




    /**
     * Web端管理员添加优惠券
     * @param couponAddRequest  优惠券添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加优惠券", description = "参数：优惠券添加请求体，权限：管理员(admin, boss)，方法名：addCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addCoupon(@RequestBody CouponAddRequest couponAddRequest) {
        if (couponAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Coupon coupon = new Coupon();
        BeanUtils.copyProperties(couponAddRequest, coupon);
        // 填充优惠券有效开始日期
        String startTime = DateUtil.format(DateUtil.date(), "yyyy-MM-dd 00:00:00");
        coupon.setStartTime(startTime);

        // 比较优惠券有效截止日期
        String endTime = coupon.getEndTime();
        String currentTime = DateUtil.now();
        if (endTime.compareTo(currentTime) <= 0) {
            coupon.setStatus("已过期");
        } else {
            coupon.setStatus("可用");
        }
        // 校验
        couponService.validCoupon(coupon, false);
        boolean result = couponService.save(coupon);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券添加失败");

        // 向消息队列中发送优惠券创建的消息
        couponService.sendCouponCreateMessage(coupon);

        return ResultUtils.success(true);
    }



    /**
     * Web端管理员更新优惠券
     * @param couponUpdateRequest   优惠券更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新优惠券", description = "参数：优惠券更新请求体，权限：管理员(admin, boss)，方法名：updateCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateCoupon(@RequestBody CouponUpdateRequest couponUpdateRequest) {
        if (couponUpdateRequest == null || couponUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponUpdateRequest.getId();
        Coupon coupon = couponService.getById(id);
        ThrowUtils.throwIf(coupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");
        BeanUtils.copyProperties(couponUpdateRequest, coupon);

        // 比较优惠券有效截止日期
        String endTime = coupon.getEndTime();
        String currentTime = DateUtil.now();
        if (endTime.compareTo(currentTime) <= 0) {
            coupon.setStatus("已过期");
        } else {
            coupon.setStatus("可用");
        }

        // 校验
        couponService.validCoupon(coupon, true);
        boolean result = couponService.saveOrUpdate(coupon);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券更新失败");

        // 向消息队列中发送优惠券创建的消息
        couponService.sendCouponCreateMessage(coupon);

        return ResultUtils.success(true);
    }



    /**
     * Web端管理员删除优惠券
     * @param couponDeleteRequest   优惠券删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "Web端管理员删除优惠券", description = "参数：优惠券删除请求体，权限：管理员(admin, boss)，方法名：deleteCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteCoupon(@RequestBody CommonRequest couponDeleteRequest) {
        if (couponDeleteRequest == null || couponDeleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponDeleteRequest.getId();
        boolean result = couponService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券删除失败");
        return ResultUtils.success(true);
    }



    /**
     * Web端管理员批量删除优惠券
     * @param commonBatchRequest   优惠券删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/del/Batch")
    @Operation(summary = "Web端管理员批量删除优惠券", description = "参数：优惠券删除请求体，权限：管理员(admin, boss)，方法名：deleteCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteCoupon(@RequestBody CommonBatchRequest commonBatchRequest) {
        if (commonBatchRequest == null || CollectionUtils.isEmpty(commonBatchRequest.getIdList())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数为null或数组为空");
        }
        List<Long> idList = commonBatchRequest.getIdList();
        boolean result = couponService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券批量删除失败");
        return ResultUtils.success(true);
    }



    /**
     * Web端管理员分页查询优惠券
     * @param couponQueryRequest 优惠券查询请求体
     * @return 优惠券列表
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询优惠券", description = "参数：优惠券查询请求体, 排序字段 sortField: conditionAmount requirePoints totalNum residueNum limitNum id，权限：管理员(admin, boss)，方法名：listCouponVOByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<Coupon>> listCouponVOByPage(@RequestBody CouponQueryRequest couponQueryRequest) {
        if (couponQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 对优惠券是否过期预处理
        long current = couponQueryRequest.getCurrent();
        long pageSize = couponQueryRequest.getPageSize();
        Page<Coupon> page = couponService.page(new Page<>(current, pageSize),
                couponService.getQueryWrapper(couponQueryRequest));
        return ResultUtils.success(page);
    }





    /**
     * 小程序端用户积分兑换优惠券
     * @param commonRequest 兑换记录添加请求体
     * @return
     */
    @PostMapping("/exchange")
    @Operation(summary = "小程序端用户积分兑换优惠券", description = "参数：兑换记录添加请求体，权限：所有人，方法名：pointsExchangeCoupon")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> pointsExchangeCoupon(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        // 获取当前用户信息
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Integer points = loginUser.getPoints();

        // 获取当前优惠券信息
        Long id = commonRequest.getId();
        Coupon coupon = couponService.getById(id);
        ThrowUtils.throwIf(coupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");

        // 判断用户积分是否充足
        Integer requirePoints = coupon.getRequirePoints();
        if (points < requirePoints) throw new BusinessException(ErrorCode.OPERATION_ERROR, "积分不足");

        // 插入一条兑换记录
        CouponVO couponVO = new CouponVO();
        BeanUtils.copyProperties(coupon, couponVO);
        UserCoupon userCoupon = new UserCoupon();
        userCoupon.setUserId(userId);
        userCoupon.setCouponVO(couponVO);
        boolean save = userCouponService.save(userCoupon);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "兑换记录插入失败");

        // 向消息队列中发送用户优惠券创建的消息
        couponService.sendUserCouponCreateMessage(userCoupon, coupon);

        // 更新用户积分
        loginUser.setPoints(loginUser.getPoints() - requirePoints);
        boolean isSuccess = userService.updateById(loginUser);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR, "更新用户积分信息失败");

        return ResultUtils.success(true, "兑换成功");
    }





    /**
     * 小程序端用户根据状态查看优惠券
     * @param commonStringRequest 优惠券状态
     * @return 当前用户拥有的优惠券列表
     */
    @PostMapping("/list/my")
    @Operation(summary = "小程序端用户查看优惠券", description = "参数：无，权限：所有人，方法名：listMyCoupon")
    public BaseResponse<List<UserCouponVO>> listMyCoupon(@RequestBody CommonStringRequest commonStringRequest, HttpServletRequest request) {
        if (commonStringRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        String status = commonStringRequest.getType();
        if (StringUtils.isBlank(status)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Map<String, Object> fieldConditions = new HashMap<>();
        fieldConditions.put("userId", userId);
        fieldConditions.put("isUsed", 0);
        List<UserCoupon> userCouponList = commonService.findByFieldEqTargetFields(fieldConditions, userCouponService);
        userCouponList = userCouponList.stream().filter(userCoupon -> userCoupon.getCouponVO().getStatus().equals(status)).toList();

        List<UserCouponVO> userCouponVOS = commonService.convertList(userCouponList, UserCouponVO.class);
        return ResultUtils.success(userCouponVOS);
    }


    /**
     * 小程序端用户查看商城优惠券
     * @return 所有可兑换的优惠券列表
     */
    @GetMapping("/list/all")
    @Operation(summary = "小程序端用户查看商城优惠券", description = "参数：无，权限：所有人，方法名：listAllCoupon")
    public BaseResponse<List<CouponVO>> listAllCoupon() {
        QueryWrapper<Coupon> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", "可用");
        List<Coupon> coupons = couponService.list(queryWrapper);
        List<CouponVO> couponVOS = commonService.convertList(coupons, CouponVO.class);
        return ResultUtils.success(couponVOS);
    }


    /**
     * 小程序端用户根据id查看优惠券详情
     * @param commonRequest 优惠券id
     * @return 优惠券详情信息
     */
    @PostMapping("/list/id")
    @Operation(summary = "小程序端用户根据id查看优惠券详情", description = "参数：优惠券id，权限：所有人，方法名：listCouponById")
    public BaseResponse<CouponVO> listCouponById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        Coupon coupon = couponService.getById(id);
        ThrowUtils.throwIf(coupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");
        CouponVO couponVO = new CouponVO();
        BeanUtils.copyProperties(coupon, couponVO);
        return ResultUtils.success(couponVO);
    }



    /**
     * Web端管理员上(下)架优惠券
     * @param commonRequest 优惠券id
     * @return 是否更新成功
     */
    @PostMapping("/shelves")
    @Operation(summary = "Web端管理员上(下)架优惠券", description = "参数：优惠券id，权限：管理员(admin, boss)，方法名：deleteExchangeRecord")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateCouponShelvesStatus(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        Coupon coupon = couponService.getById(id);
        ThrowUtils.throwIf(coupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");

        Integer status = coupon.getIsShelves() == 0 ? 1 : 0;
        UpdateWrapper<Coupon> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isShelves", status);
        boolean update = couponService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "上架状态更新失败");
        return ResultUtils.success(true);
    }




    /**
     * 小程序端用户查看(不)能够使用的优惠券
     * @param usableCouponQueryRequest 可用优惠券请求体
     * @return 优惠券详情信息
     */
    @PostMapping("/list/use")
    @Operation(summary = "小程序端用户查看(不)能够使用的优惠券", description = "参数：可用优惠券请求体，权限：所有人，方法名：listUsableCoupon")
    public BaseResponse<List<UserCouponVO>> listUsableCoupon(@RequestBody UsableCouponQueryRequest usableCouponQueryRequest, HttpServletRequest request) {
        if (usableCouponQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        BigDecimal currentAmount = usableCouponQueryRequest.getCurrentAmount();
        if (currentAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "订单金额参数错误");
        }
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Map<String, Object> fieldConditions = new HashMap<>();
        fieldConditions.put("userId", userId);
        fieldConditions.put("isUsed", 0);
        List<UserCoupon> userCouponList = commonService.findByFieldEqTargetFields(fieldConditions, userCouponService);
        userCouponList = userCouponList.stream().filter(userCoupon -> userCoupon.getCouponVO().getStatus().equals("可用")).toList();

        // 获取(不)可使用的优惠券
        List<UserCoupon> userCoupons = userCouponList.stream().filter(userCoupon -> {
            CouponVO couponVO = userCoupon.getCouponVO();
            BigDecimal standardAmount = couponVO.getStandardAmount();
            Boolean isAvailable = usableCouponQueryRequest.getIsAvailable();
            int result = currentAmount.compareTo(standardAmount);
            return isAvailable == (result >= 0);
        }).toList();

        List<UserCouponVO> userCouponVOS = commonService.convertList(userCoupons, UserCouponVO.class);
        return ResultUtils.success(userCouponVOS);
    }






    /**
     * 小程序端用户查看当前拥有的优惠券（包括可用的和不可用的）
     * @param couponPriceQueryRequest 订单金额
     * @return （不）可用的的优惠券里列表
     */
    @PostMapping("/list/all/own")
    @Operation(summary = "小程序端用户查看当前拥有的优惠券（包括可用的和不可用的）", description = "参数：优惠券id，权限：所有人，方法名：listAllOwnCoupon")
    public BaseResponse<Map<String, List<UserCouponVO>>> listAllOwnCoupon(@RequestBody CouponPriceQueryRequest couponPriceQueryRequest, HttpServletRequest request) {
        if (couponPriceQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        BigDecimal currentAmount = couponPriceQueryRequest.getCurrentAmount();
        if (currentAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "订单金额参数错误");
        }
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Map<String, Object> fieldConditions = new HashMap<>();
        fieldConditions.put("userId", userId);
        fieldConditions.put("isUsed", 0);
        List<UserCoupon> userCouponList = commonService.findByFieldEqTargetFields(fieldConditions, userCouponService);
        userCouponList = userCouponList.stream().filter(userCoupon -> userCoupon.getCouponVO().getStatus().equals("可用")).toList();

        List<UserCoupon> enabledToUseCouponList = new ArrayList<>();
        List<UserCoupon> noEnabledToUserCouponList = new ArrayList<>();
        for (UserCoupon userCoupon : userCouponList) {
            CouponVO couponVO = userCoupon.getCouponVO();
            BigDecimal standardAmount = couponVO.getStandardAmount();
            int result = currentAmount.compareTo(standardAmount);
            if (result >= 0) enabledToUseCouponList.add(userCoupon);
            else noEnabledToUserCouponList.add(userCoupon);
        }
        List<UserCouponVO> enabledToUseCouponVOList = commonService.convertList(enabledToUseCouponList, UserCouponVO.class);
        List<UserCouponVO> noEnabledToUserCouponVOList = commonService.convertList(noEnabledToUserCouponList, UserCouponVO.class);

        Map<String, List<UserCouponVO>> userCouponMap = new HashMap<>();
        userCouponMap.put("可用", enabledToUseCouponVOList);
        userCouponMap.put("不可用", noEnabledToUserCouponVOList);

        return ResultUtils.success(userCouponMap);
    }






}
