package com.cultural.heritage.service.book.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.constant.MqConstant;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.AdvanceOrderMapper;
import com.cultural.heritage.model.dto.advanceOrder.AdvanceOrderQueryRequest;
import com.cultural.heritage.model.dto.snapshot.ContactsSnapshot;
import com.cultural.heritage.model.dto.snapshot.PhotoProductsSnapshot;
import com.cultural.heritage.model.entity.AdvanceOrder;
import com.cultural.heritage.service.book.AdvanceOrderService;
import com.cultural.heritage.utils.MultiDelayMessage;
import com.cultural.heritage.utils.SqlUtils;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

@Service
public class AdvanceOrderServiceImpl extends ServiceImpl<AdvanceOrderMapper, AdvanceOrder> implements AdvanceOrderService {


    @Resource
    private RabbitTemplate rabbitTemplate;


    /**
     * 校验
     */
    @Override
    public void validAdvanceOrder(AdvanceOrder advanceOrder) {
        PhotoProductsSnapshot photoProductsSnapshot = advanceOrder.getPhotoProductsSnapshot();
        BigDecimal totalAmount = advanceOrder.getTotalAmount();
        ContactsSnapshot contactsSnapshot = advanceOrder.getContactsSnapshot();

        String specificDate = advanceOrder.getSpecificDate();
        String timePoint = advanceOrder.getTimePoint();
        Integer isIndoors = advanceOrder.getIsIndoors();
        Integer isMakeup = advanceOrder.getIsMakeup();
        String orderStatus = advanceOrder.getOrderStatus();
        Long userId = advanceOrder.getUserId();

        if (ObjectUtils.isEmpty(totalAmount) || totalAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "定金参数错误");
        }
        if (ObjectUtils.anyNull(contactsSnapshot, photoProductsSnapshot, isIndoors, isMakeup, userId)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为null");
        }
        if (StringUtils.isAnyBlank(specificDate, timePoint, orderStatus)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为空");
        }
    }


    /**
     * 获取查询条件
     */
    @Override
    public QueryWrapper<AdvanceOrder> getQueryWrapper(AdvanceOrderQueryRequest advanceOrderQueryRequest) {
        Long id = advanceOrderQueryRequest.getId();
        BigDecimal minAmount = advanceOrderQueryRequest.getMinAmount();
        BigDecimal maxAmount = advanceOrderQueryRequest.getMaxAmount();
        String startTime = advanceOrderQueryRequest.getStartTime();
        String endTime = advanceOrderQueryRequest.getEndTime();
        String orderStatus = advanceOrderQueryRequest.getOrderStatus();
        String sortField = advanceOrderQueryRequest.getSortField();
        String sortOrder = advanceOrderQueryRequest.getSortOrder();
        String createLeftTime = advanceOrderQueryRequest.getCreateLeftTime();
        String createRightTime = advanceOrderQueryRequest.getCreateRightTime();

        QueryWrapper<AdvanceOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(id != null, "id", id);
        queryWrapper.ge(ObjectUtils.isNotEmpty(minAmount), "totalAmount", minAmount);
        queryWrapper.le(ObjectUtils.isNotEmpty(maxAmount), "totalAmount", maxAmount);
        queryWrapper.between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime), "specificDate", startTime, endTime);
        queryWrapper.between(StringUtils.isNotBlank(createLeftTime) && StringUtils.isNotBlank(createRightTime), "createTime", createLeftTime, createRightTime);
        queryWrapper.eq(StringUtils.isNotBlank(orderStatus), "orderStatus", orderStatus);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }


    /**
     * 向消息队列中发送订单创建的消息
     */
    @Override
    public void sendAdvanceOrderMessage(Long orderId) {
        // 复制 DELAY_MILLIS 到一个新的 ArrayList
        List<Long> newDelayMillis = new ArrayList<>(MqConstant.DELAY_MILLIS);
        // 延迟检查订单状态信息
        MultiDelayMessage<Long> msg = new MultiDelayMessage<>(orderId, newDelayMillis);
        int delayValue = msg.removeNextDelay().intValue();
        rabbitTemplate.convertAndSend(MqConstant.DELAY_EXCHANGE,
                MqConstant.DELAY_ADVANCE_ROUTING_KEY, msg, message -> {
                    // 添加延迟消息属性
                    message.getMessageProperties().setDelay(delayValue);
                    return message;
                });
    }



    /**
     * 校验当前订单时间是否被预约过
     */
    @Override
    public void validBookingTime(String specificDate, String timePoint) {
        String time = timePoint + ":00";
        DateTime targetTime = DateUtil.parse(specificDate + ' ' + time, "yyyy-MM-dd HH:mm:ss");
        DateTime now = DateUtil.date();
        DateTime currentTime = DateUtil.parse(now.toString("yyyy-MM-dd HH:mm:ss"), "yyyy-MM-dd HH:mm:ss");
        int result = currentTime.compareTo(targetTime);
        ThrowUtils.throwIf(result >= 0, ErrorCode.OPERATION_ERROR, "当前预约时间已过期");

        QueryWrapper<AdvanceOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("specificDate", specificDate)
                    .eq("timePoint", timePoint)
                    .and(wrapper -> wrapper.eq("orderStatus", OrderStatusConstant.PENDING_PAYMENT)
                            .or().eq("orderStatus", OrderStatusConstant.PENDING_SHIPMENT));
        AdvanceOrder order = this.getOne(queryWrapper);
        ThrowUtils.throwIf(order != null, ErrorCode.OPERATION_ERROR, "当前时间已约满");
    }
}
